@@ -2,7 +2,7 @@ import copy
 import inspect
 import logging
 from time import time
-from confluent_kafka import Producer, Consumer, TopicPartition as TopicPartitionOffset, OFFSET_BEGINNING, TIMESTAMP_NOT_AVAILABLE
+from confluent_kafka import Producer, Consumer, TopicPartition as TopicPartitionOffset, OFFSET_BEGINNING, OFFSET_END, TIMESTAMP_NOT_AVAILABLE, KafkaException, KafkaError
 from kafka.future import Future
 from kafkacrypto.exceptions import KafkaCryptoWrapperError
 from collections import namedtuple
@@ -30,32 +30,48 @@ class FutureRecordMetadata(Future):
     self.value_len = value_len
 
   def base_callback(self, err, msg):
-    if err != None:
-      self.failure(err)
+    self._producer._log.debug("Entering base callback with err=%s and msg=%s.", str(err), str(msg))
+    if err is not None:
+      self._producer._log.debug("Callback failed (non-None err).")
+      super().failure(KafkaException(err))
     elif msg is None:
-      self.failure('null msg')
+      self._producer._log.debug("Callback failed (None msg).")
+      super().failure(KafkaException(KafkaError.UNKNOWN,'null msg'))
     elif msg.error() != None:
-      self.failure(msg.error())
+      self._producer._log.debug("Callback failed (non-None msg.error).")
+      super().failure(KafkaException(msg.error()))
     else:
       # success
+      self._producer._log.debug("Callback success.")
       metadata = RecordMetadata(msg.topic(), msg.partition(), TopicPartition(msg.topic(), msg.partition()),
-                                msg.offset(), msg.timestamp()[1] if msg.timestamp()[0]!=TIMESTAMP_NOT_AVAILABLE else int(time()*1000), 
+                                msg.offset(), msg.timestamp()[1] if msg.timestamp()[0]!=TIMESTAMP_NOT_AVAILABLE else int(time()*1000),
                                 None, self.key_len, self.value_len, -1)
-      self.success(metadata)
+      super().success(metadata)
+    self._producer._log.debug("Finished with base callback with err=%s and msg=%s.", str(err), str(msg))
 
-  def get(self, timeout=None, timeout_jiffy=0.1):
+  def get(self, timeout="default", timeout_jiffy=0.1):
+    # timeout = None is infinite timeout, for compatibility with kafka-python's FutureRecordMetadata
+    # timeout = "default" (or any non-number) should never be passed by callers, but is used here
+    #           to indicate use of configured default.
     if timeout is None:
       last_time = 9223372036854775807
       timeout = last_time-time()
+    elif timeout == "default":
+      timeout = self._producer.config['produce_timeout']
+      last_time = time()+timeout
     else:
       last_time = time()+timeout
+    self._producer._log.debug("Entering FutureRecordMetadata get with timeout=%s.", str(timeout))
+    orig_timeout = timeout
     while not self.is_done and timeout>0:
       self._producer.poll(min(timeout,timeout_jiffy))
       timeout = last_time-time()
+      self._producer._log.debug("FutureRecordMetadata polling done, remaining timeout=%s.", str(timeout))
     if not self.is_done:
-      raise FutureTimeoutError("Timeout after waiting for %s secs." % (timeout,))
-    if self.failed():
+      raise FutureTimeoutError("Timeout after waiting for %s secs." % (orig_timeout,))
+    if super().failed():
       raise self.exception
+    self._producer._log.debug("Finished with FutureRecordMetadata get")
     return self.value
 
 
@@ -102,9 +118,9 @@ class KafkaConsumer(Consumer):
                }
   CONFIG_MAP_LOCAL = { 'max_poll_records': 500,
                      }
-  CONFIG_MAP_NULL = [ 'ssl_context', 
-                      'socket_options', 
-                      'default_offset_commit_callback', 
+  CONFIG_MAP_NULL = [ 'ssl_context',
+                      'socket_options',
+                      'default_offset_commit_callback',
                       'consumer_timeout_ms',
                       'ssl_check_hostname',
                       'api_version',
@@ -157,6 +173,7 @@ class KafkaConsumer(Consumer):
       self.subscribe(topics)
 
   def commit(self, offsets=None):
+    self._log.debug("Executing Consumer commit.")
     if offsets==None:
       return super().commit()
     else:
@@ -166,6 +183,7 @@ class KafkaConsumer(Consumer):
       return super().commit(offsets=offs)
 
   def subscribe(self, topics=None, pattern=None, listener=None):
+    self._log.info("Executing Consumer subscribe with topics=%s, pattern=%s, listener=%s", str(topics), str(pattern), str(listener))
     if not topics is None:
       topics = copy.copy(topics)
     else:
@@ -182,7 +200,11 @@ class KafkaConsumer(Consumer):
 
   def poll(self, timeout_ms=0, max_records=None):
     rvm = {}
-    msgs = super().consume(self.config['max_poll_records'],timeout_ms/1000.0)
+    if (max_records is None) or max_records <= 0 or max_records > self.config['max_poll_records']:
+      max_records = self.config['max_poll_records']
+    self._log.debug("Entering Consumer poll with timeout_ms=%s and max_records=%s", str(timeout_ms), str(max_records))
+    msgs = super().consume(max_records,timeout_ms/1000.0)
+    self._log.debug("Consumer poll consume complete.")
     for msg in msgs:
       if not (msg is None) and msg.error() is None:
         rvk = TopicPartition(msg.topic(),msg.partition())
@@ -194,12 +216,14 @@ class KafkaConsumer(Consumer):
     return rvm
 
   def assign(self, partitions):
+    self._log.info("Executing Consumer assign with partitions=%s.", str(partitions))
     tps = []
     for tp in partitions:
       tps.append(TopicPartitionOffset(tp.topic, tp.partition))
     return super().assign(tps)
 
   def assign_and_seek(self, partoffs):
+    self._log.info("Executing Consumer assign_and_seek with partoffs=%s.", str(partoffs))
     tps = []
     for tpo in partoffs:
       if (tpo.offset > 0):
@@ -209,15 +233,29 @@ class KafkaConsumer(Consumer):
     return super().assign(tps)
 
   def seek(self, tp, offset):
+    self._log.debug("Executing Consumer seek.")
     return super().seek(TopicPartitionOffset(tp.topic, tp.partition, offset))
 
-  def seek_to_beginning(self):
-    for tp in self.assignment():
+  def seek_to_beginning(self, tps=None):
+    self._log.debug("Executing Consumer seek_to_beginning.")
+    if tps is None or len(tps) < 1:
+      tps = self.assignment()
+    for tp in tps:
       try:
         self.seek(tp, OFFSET_BEGINNING)
       except:
         pass
 
+  def seek_to_end(self, tps=None):
+    self._log.debug("Executing Consumer seek_to_end.")
+    if tps is None or len(tps) < 1:
+      tps = self.assignment()
+    for tp in tps:
+      try:
+        self.seek(tp, OFFSET_END)
+      except:
+        pass
+
   def __iter__(self):
     return self
 
@@ -239,7 +277,7 @@ class KafkaProducer(Producer):
                  'ssl_crlfile': 'ssl.crl.location',
                  'ssl_password': 'ssl.key.password',
                  'ssl_ciphers': 'ssl.cipher.suites',
-                 'max_request_size': 'message.max.bytes', # this maps the hard limit functionality appropriately                                                 
+                 'max_request_size': 'message.max.bytes', # this maps the hard limit functionality appropriately
                  'bootstrap_servers': 'bootstrap.servers',
                  'client_id': 'client.id',
                  'group_id': 'group.id',
@@ -261,7 +299,9 @@ class KafkaProducer(Producer):
                  'sasl_plain_password': 'sasl.password',
                  'sasl_kerberos_service_name': 'sasl.kerberos.service.name',
                }
-  CONFIG_MAP_NULL = [ 'ssl_context', 
+  CONFIG_MAP_LOCAL = { 'produce_timeout': 30, # in s
+                     }
+  CONFIG_MAP_NULL = [ 'ssl_context',
                       'socket_options',
                       'batch_size',
                       'partitioner',
@@ -285,7 +325,13 @@ class KafkaProducer(Producer):
     self.config = {}
     self.ks = lambda topic, _bytes: bytes(_bytes)
     self.vs = lambda topic, _bytes: bytes(_bytes)
+    for k in self.CONFIG_MAP_LOCAL.keys():
+      self.config[k] = self.CONFIG_MAP_LOCAL[k]
     for k in self.raw_config.keys():
+      # adjust default produce_timeout if not separately set, and
+      # other parameters are set that require it to adjust.
+      if (k in ['delivery.timeout.ms', 'message.timeout.ms', 'transaction.timeout.ms']) and ('produce_timeout' not in self.raw_config.keys()):
+        self.config['produce_timeout'] = self.raw_config[k]/1000.0
       if (k == 'key_serializer'):
         if not (self.raw_config[k] is None) and (not hasattr(self.raw_config[k], 'serialize') or not inspect.isroutine(self.raw_config[k].serialize)):
           self.ks = lambda topic, _bytes: self.raw_config[k](_bytes)
@@ -301,6 +347,8 @@ class KafkaProducer(Producer):
       elif (k in self.CONFIG_MAP.keys()):
         if not (self.raw_config[k] is None):
           self.cf_config[self.CONFIG_MAP[k]] = self.raw_config[k]
+      elif (k in self.CONFIG_MAP_LOCAL):
+        self.config[k] = self.raw_config[k]
       elif (k in self.CONFIG_MAP_NULL):
         self._log.warning("Warning: Unsupported kafka-python parameter passed to confluent wrapper: %s, %s", k, self.raw_config[k])
       else:
@@ -310,7 +358,7 @@ class KafkaProducer(Producer):
         self.config[oldk] = self.cf_config[newk]
     super().__init__(self.cf_config)
 
-  def flush(self, timeout=None):
+  def flush(self, timeout="default", timeout_jiffy=0.1):
     # librdkafka 1.8.0 changed the behavior of flush() to ignore linger_ms and
     # immediately attempt to send messages. Unfortunately, that change added
     # a call to rd_kafka_all_brokers_wakeup , which seems to cause a hang of
@@ -319,21 +367,51 @@ class KafkaProducer(Producer):
     #
     # This eventually results in non-sensical exceptions being thrown. We
     # fix it here by implementing flush as sucessive polling directly.
-    left = len(self)
-    jiffy = 0.1
+    #
+    # It is highly recommended this be used with a timeout, since otherwise
+    # with high message production rates and other poll-callers winning the
+    # race for calling callbacks, this will never complete.
+    #
+    # timeout = None is infinite timeout
+    # timeout = "default" (or any non-number) should never be passed by callers,
+    #           but is used here to indicate use of configured default.
+    #
     if timeout is None:
+      # confluent_kafka uses -1 for infinite timeout
+      timeout = -1
+    elif timeout == "default":
+      timeout = self.config['produce_timeout']
+    left = len(self)
+    initial = left
+    self._log.debug("Entering Producer flush with timeout=%s and left=%s.", str(timeout), str(left))
+    if timeout<0:
       while left > 0:
-        con = self.poll(jiffy)
-        left = left - con
-    else:
+        con = self.poll(timeout_jiffy)
+        initial -= con
+        left = min([len(self),initial])
+        self._log.debug("Producer flush poll cycle complete, left=%s.", str(left))
+    elif left > 0:
       con = self.poll(timeout)
-      left = left - con
+      initial -= con
+      left = min([len(self),initial])
+      self._log.debug("Producer flush single poll cycle complete, left=%s.", str(left))
+    if max([left,0]) != 0:
+      self._log.info("Producer flush complete, but messages left=%s greater than zero.", str(max([left,0])))
+    else:
+      self._log.debug("Producer flush complete.")
     return max([left,0])
 
   def poll(self, timeout=0):
+    # timeout = 0 means nowait (default)
+    # timeout = None means infinite timeout
+    self._log.debug("Executing Producer poll with timeout=%s.", str(timeout))
+    if timeout is None:
+      # confluent_kafka uses -1 for infinite timeout
+      timeout = -1
     return super().poll(timeout)
 
   def send(self, topic, value=None, key=None, headers=None, partition=0, timestamp_ms=None):
+    self._log.debug("Executing Producer send to topic=%s, with value=%s, key=%s, headers=%s, partition=%s, timestamp_ms=%s.", str(topic), str(value), str(key), str(headers), str(partition), str(timestamp_ms))
     if key!=None:
       key = self.ks(topic, key)
     if value!=None:
