@@ -177,7 +177,7 @@ class KafkaCrypto(KafkaCryptoBase):
                 else:
                   self._logger.info("No keys for root=%s to send to new receiver.", root)
             except Exception as e:
-              self._parent._logger.warning("".join(format_exception_shim(etype=type(e), value=e, tb=e.__traceback__)))
+              self._parent._logger.warning("".join(format_exception_shim(e)))
           elif topic[-len(self.TOPIC_SUFFIX_KEYS):] == self.TOPIC_SUFFIX_KEYS:
             root = topic[:-len(self.TOPIC_SUFFIX_KEYS)]
             # New key(s)
@@ -226,9 +226,12 @@ class KafkaCrypto(KafkaCryptoBase):
             # unknown object
             log_limited(self._logger.warning, "Unknown topic type in message: %s", msg)
         self._lock.release()
-  
+
       # Flush producer
-      self._kp.flush()
+      try:
+        self._kp.flush(timeout=self.MGMT_FLUSH_TIME)
+      except Exception as e:
+        self._parent._logger.warning("".join(format_exception_shim(e)))
 
       # Second, deal with subscription changes
       self._lock.acquire()
@@ -266,8 +269,8 @@ class KafkaCrypto(KafkaCryptoBase):
               if not (k is None) and not (v is None):
                 self._logger.info("Sending new subscribe request for root=%s, msgkey=%s", root, k)
                 self._kp.send(root + self.TOPIC_SUFFIX_SUBS, key=k, value=v)
-                if not self._cryptoexchange.valid_spk_chain():
-                  # if using default/temp ROT, send directly as well
+                if self._cryptoexchange.direct_request_spk_chain():
+                  # if it may succeed, send directly as well
                   self._kp.send(root + self.TOPIC_SUFFIX_REQS, key=k, value=v)
                 self._subs_last[root] = [time(),kis]
               else:
@@ -282,7 +285,10 @@ class KafkaCrypto(KafkaCryptoBase):
       self._lock.release()
 
       # Flush producer
-      self._kp.flush()
+      try:
+        self._kp.flush(timeout=self.MGMT_FLUSH_TIME)
+      except Exception as e:
+        self._parent._logger.warning("".join(format_exception_shim(e)))
 
       # Fourth, periodically increment ratchet and prune old keys
       self._logger.debug("Checking ratchet time.")
@@ -363,7 +369,7 @@ class KafkaCrypto(KafkaCryptoBase):
         key,nonce = gen.generate()
         msg = b'\x01' + msgpack.packb([keyidx,salt,pysodium.crypto_secretbox(value,nonce,key)], use_bin_type=True)
       except Exception as e:
-        self._parent._logger.warning("".join(format_exception_shim(etype=type(e), value=e, tb=e.__traceback__)))
+        self._parent._logger.warning("".join(format_exception_shim(e)))
       finally:
         self._parent._lock.release()
       return msg
@@ -389,7 +395,7 @@ class KafkaCrypto(KafkaCryptoBase):
         if (len(msg) != 3):
           raise KafkaCryptoSerializeError("Malformed Message!")
       except Exception as e:
-        self._parent._logger.debug("".join(format_exception_shim(etype=type(e), value=e, tb=e.__traceback__)))
+        self._parent._logger.debug("".join(format_exception_shim(e)))
         return KafkaCryptoMessage.fromBytes(bytes_,deser=self,topic=topic)
       ki = msg[0]
       salt = msg[1]
