@@ -37,7 +37,7 @@ class CryptoKey(object):
     contents = None
     while len(data) and contents is None:
       try:
-        contents = msgpack.unpackb(data)
+        contents = msgpack.unpackb(data,raw=True)
       except msgpack.exceptions.ExtraData:
         data = data[:-1]
     if len(data) != datalen:
@@ -59,20 +59,24 @@ class CryptoKey(object):
     #
     # returns the public key of a new ephemeral encryption key for the specified topic
     #
-    if (isinstance(topic,(str))):
-      topic = bytes(topic, 'utf-8')
-    if (isinstance(usage,(str))):
-      usage = bytes(usage, 'utf-8')
+    if (isinstance(topic,(bytes,bytearray))):
+      self._logger.debug("passed a topic in bytes (should be string)")
+      topic = topic.decode('utf-8')
+    if (isinstance(usage,(bytes,bytearray))):
+      self._logger.debug("passed a usage in bytes (should be string)")
+      usage = usage.decode('utf-8')
     with self.__eklock:
       self.__generate_esk(topic, usage)
       return self.__epk[topic][usage]
 
   def use_epk(self, topic, usage, pks, clear=True):
     rv = []
-    if (isinstance(topic,(str))):
-      topic = bytes(topic, 'utf-8')
-    if (isinstance(usage,(str))):
-      usage = bytes(usage, 'utf-8')
+    if (isinstance(topic,(bytes,bytearray))):
+      self._logger.debug("passed a topic in bytes (should be string)")
+      topic = topic.decode('utf-8')
+    if (isinstance(usage,(bytes,bytearray))):
+      self._logger.debug("passed a usage in bytes (should be string)")
+      usage = usage.decode('utf-8')
     with self.__eklock:
       if not topic in self.__esk or not usage in self.__esk[topic]:
         return rv
@@ -111,5 +115,5 @@ class CryptoKey(object):
     pk,sk = pysodium.crypto_sign_keypair()
     self._logger.warning("  Public key: %s", pysodium.crypto_sign_sk_to_pk(sk).hex())
     with open(file, "wb") as f:
-      f.write(msgpack.packb([sk,pysodium.randombytes(pysodium.crypto_secretbox_KEYBYTES)]))
+      f.write(msgpack.packb([sk,pysodium.randombytes(pysodium.crypto_secretbox_KEYBYTES)], use_bin_type=True))
     self._logger.warning("  CryptoKey Initialized. Provisioning required for successful operation.")
